/*      $NetBSD: rmd128.c,v 1.1.1.1 2001/03/06 11:21:05 agc Exp $    */
/*      $RoughId: rmd128.c,v 1.2 2001/07/13 19:49:10 knu Exp $       */
/*      $Id: rmd128.c,v 1.1 2001/07/13 20:06:14 knu Exp $    */

/********************************************************************\
 *
 *      FILE:     rmd128.c
 *
 *      CONTENTS: A sample C-implementation of the RIPEMD-128
 *                hash-function.
 *      TARGET:   any computer with an ANSI C compiler
 *
 *      AUTHOR:   Antoon Bosselaers, ESAT-COSIC
 *                (Arranged for libc by Todd C. Miller)
 *      DATE:     1 March 1996
 *      VERSION:  1.0
 *
 *      Copyright (c) Katholieke Universiteit Leuven
 *      1996, All Rights Reserved
 *
\********************************************************************/

/*typedef unsigned int uint32_t;
typedef unsigned char uint8_t;
#include <stddef.h>*/

#include "rmd128.h"

#ifndef lint
/* __RCSID("$NetBSD: rmd128.c,v 1.1.1.1 2001/03/06 11:21:05 agc Exp $"); */
#endif  /* not lint */

/* header files */

#ifdef HAVE_SYS_ENDIAN_H_
#include <sys/endian.h>
#endif

#ifdef HAVE_MACHINE_ENDIAN_H_
#include <machine/endian.h>
#endif

/* #include "namespace.h" */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _DIAGASSERT
#define _DIAGASSERT(cond)       assert(cond)
#endif


/********************************************************************/

/* macro definitions */


/* the eight basic operations FF() through III() */
#define FF128(a, b, c, d, x, s)        {\
      (a) += F((b), (c), (d)) + (x);\
      (a) = ROL((a), (s));\
   }
#define GG128(a, b, c, d, x, s)        {\
      (a) += G((b), (c), (d)) + (x) + 0x5a827999UL;\
      (a) = ROL((a), (s));\
   }
#define HH128(a, b, c, d, x, s)        {\
      (a) += H((b), (c), (d)) + (x) + 0x6ed9eba1UL;\
      (a) = ROL((a), (s));\
   }
#define II128(a, b, c, d, x, s)        {\
      (a) += I((b), (c), (d)) + (x) + 0x8f1bbcdcUL;\
      (a) = ROL((a), (s));\
   }
#define FFF128(a, b, c, d, x, s)        {\
      (a) += F((b), (c), (d)) + (x);\
      (a) = ROL((a), (s));\
   }
#define GGG128(a, b, c, d, x, s)        {\
      (a) += G((b), (c), (d)) + (x) + 0x6d703ef3UL;\
      (a) = ROL((a), (s));\
   }
#define HHH128(a, b, c, d, x, s)        {\
      (a) += H((b), (c), (d)) + (x) + 0x5c4dd124UL;\
      (a) = ROL((a), (s));\
   }
#define III128(a, b, c, d, x, s)        {\
      (a) += I((b), (c), (d)) + (x) + 0x50a28be6UL;\
      (a) = ROL((a), (s));\
   }

/********************************************************************/

void
RMD128_Init(RMD128_CTX *context)
{

        _DIAGASSERT(context != NULL);

        /* ripemd-128 initialization constants */
        context->state[0] = 0x67452301U;
        context->state[1] = 0xefcdab89U;
        context->state[2] = 0x98badcfeU;
        context->state[3] = 0x10325476U;
        context->state[4] = 0xc3d2e1f0U;
        context->length[0] = context->length[1] = 0;
        context->buflen = 0;
}

/********************************************************************/

void
RMD128_Transform(uint32_t state[5], const uint32_t block[16])
{
        uint32_t aa, bb, cc, dd, ee;
        uint32_t aaa, bbb, ccc, ddd, eee;

        _DIAGASSERT(state != NULL);
        _DIAGASSERT(block != NULL);

        aa = aaa = state[0];
        bb = bbb = state[1];
        cc = ccc = state[2];
        dd = ddd = state[3];

        /* round 1 */
        FF128(aa, bb, cc, dd, block[ 0], 11);
        FF128(dd, aa, bb, cc, block[ 1], 14);
        FF128(cc, dd, aa, bb, block[ 2], 15);
        FF128(bb, cc, dd, aa, block[ 3], 12);
        FF128(aa, bb, cc, dd, block[ 4],  5);
        FF128(dd, aa, bb, cc, block[ 5],  8);
        FF128(cc, dd, aa, bb, block[ 6],  7);
        FF128(bb, cc, dd, aa, block[ 7],  9);
        FF128(aa, bb, cc, dd, block[ 8], 11);
        FF128(dd, aa, bb, cc, block[ 9], 13);
        FF128(cc, dd, aa, bb, block[10], 14);
        FF128(bb, cc, dd, aa, block[11], 15);
        FF128(aa, bb, cc, dd, block[12],  6);
        FF128(dd, aa, bb, cc, block[13],  7);
        FF128(cc, dd, aa, bb, block[14],  9);
        FF128(bb, cc, dd, aa, block[15],  8);

        /* round 2 */
        GG128(aa, bb, cc, dd, block[ 7],  7);
        GG128(dd, aa, bb, cc, block[ 4],  6);
        GG128(cc, dd, aa, bb, block[13],  8);
        GG128(bb, cc, dd, aa, block[ 1], 13);
        GG128(aa, bb, cc, dd, block[10], 11);
        GG128(dd, aa, bb, cc, block[ 6],  9);
        GG128(cc, dd, aa, bb, block[15],  7);
        GG128(bb, cc, dd, aa, block[ 3], 15);
        GG128(aa, bb, cc, dd, block[12],  7);
        GG128(dd, aa, bb, cc, block[ 0], 12);
        GG128(cc, dd, aa, bb, block[ 9], 15);
        GG128(bb, cc, dd, aa, block[ 5],  9);
        GG128(aa, bb, cc, dd, block[ 2], 11);
        GG128(dd, aa, bb, cc, block[14],  7);
        GG128(cc, dd, aa, bb, block[11], 13);
        GG128(bb, cc, dd, aa, block[ 8], 12);

        /* round 3 */
        HH128(aa, bb, cc, dd, block[ 3], 11);
        HH128(dd, aa, bb, cc, block[10], 13);
        HH128(cc, dd, aa, bb, block[14],  6);
        HH128(bb, cc, dd, aa, block[ 4],  7);
        HH128(aa, bb, cc, dd, block[ 9], 14);
        HH128(dd, aa, bb, cc, block[15],  9);
        HH128(cc, dd, aa, bb, block[ 8], 13);
        HH128(bb, cc, dd, aa, block[ 1], 15);
        HH128(aa, bb, cc, dd, block[ 2], 14);
        HH128(dd, aa, bb, cc, block[ 7],  8);
        HH128(cc, dd, aa, bb, block[ 0], 13);
        HH128(bb, cc, dd, aa, block[ 6],  6);
        HH128(aa, bb, cc, dd, block[13],  5);
        HH128(dd, aa, bb, cc, block[11], 12);
        HH128(cc, dd, aa, bb, block[ 5],  7);
        HH128(bb, cc, dd, aa, block[12],  5);

        /* round 4 */
        II128(aa, bb, cc, dd, block[ 1], 11);
        II128(dd, aa, bb, cc, block[ 9], 12);
        II128(cc, dd, aa, bb, block[11], 14);
        II128(bb, cc, dd, aa, block[10], 15);
        II128(aa, bb, cc, dd, block[ 0], 14);
        II128(dd, aa, bb, cc, block[ 8], 15);
        II128(cc, dd, aa, bb, block[12],  9);
        II128(bb, cc, dd, aa, block[ 4],  8);
        II128(aa, bb, cc, dd, block[13],  9);
        II128(dd, aa, bb, cc, block[ 3], 14);
        II128(cc, dd, aa, bb, block[ 7],  5);
        II128(bb, cc, dd, aa, block[15],  6);
        II128(aa, bb, cc, dd, block[14],  8);
        II128(dd, aa, bb, cc, block[ 5],  6);
        II128(cc, dd, aa, bb, block[ 6],  5);
        II128(bb, cc, dd, aa, block[ 2], 12);

        /* parallel round 1 */
        III128(aaa, bbb, ccc, ddd, block[ 5],  8); 
        III128(ddd, aaa, bbb, ccc, block[14],  9);
        III128(ccc, ddd, aaa, bbb, block[ 7],  9);
        III128(bbb, ccc, ddd, aaa, block[ 0], 11);
        III128(aaa, bbb, ccc, ddd, block[ 9], 13);
        III128(ddd, aaa, bbb, ccc, block[ 2], 15);
        III128(ccc, ddd, aaa, bbb, block[11], 15);
        III128(bbb, ccc, ddd, aaa, block[ 4],  5);
        III128(aaa, bbb, ccc, ddd, block[13],  7);
        III128(ddd, aaa, bbb, ccc, block[ 6],  7);
        III128(ccc, ddd, aaa, bbb, block[15],  8);
        III128(bbb, ccc, ddd, aaa, block[ 8], 11);
        III128(aaa, bbb, ccc, ddd, block[ 1], 14);
        III128(ddd, aaa, bbb, ccc, block[10], 14);
        III128(ccc, ddd, aaa, bbb, block[ 3], 12);
        III128(bbb, ccc, ddd, aaa, block[12],  6);

        /* parallel round 2 */
        HHH128(aaa, bbb, ccc, ddd, block[ 6],  9);
        HHH128(ddd, aaa, bbb, ccc, block[11], 13);
        HHH128(ccc, ddd, aaa, bbb, block[ 3], 15);
        HHH128(bbb, ccc, ddd, aaa, block[ 7],  7);
        HHH128(aaa, bbb, ccc, ddd, block[ 0], 12);
        HHH128(ddd, aaa, bbb, ccc, block[13],  8);
        HHH128(ccc, ddd, aaa, bbb, block[ 5],  9);
        HHH128(bbb, ccc, ddd, aaa, block[10], 11);
        HHH128(aaa, bbb, ccc, ddd, block[14],  7);
        HHH128(ddd, aaa, bbb, ccc, block[15],  7);
        HHH128(ccc, ddd, aaa, bbb, block[ 8], 12);
        HHH128(bbb, ccc, ddd, aaa, block[12],  7);
        HHH128(aaa, bbb, ccc, ddd, block[ 4],  6);
        HHH128(ddd, aaa, bbb, ccc, block[ 9], 15);
        HHH128(ccc, ddd, aaa, bbb, block[ 1], 13);
        HHH128(bbb, ccc, ddd, aaa, block[ 2], 11);

        /* parallel round 3 */        
        GGG128(aaa, bbb, ccc, ddd, block[15],  9);
        GGG128(ddd, aaa, bbb, ccc, block[ 5],  7);
        GGG128(ccc, ddd, aaa, bbb, block[ 1], 15);
        GGG128(bbb, ccc, ddd, aaa, block[ 3], 11);
        GGG128(aaa, bbb, ccc, ddd, block[ 7],  8);
        GGG128(ddd, aaa, bbb, ccc, block[14],  6);
        GGG128(ccc, ddd, aaa, bbb, block[ 6],  6);
        GGG128(bbb, ccc, ddd, aaa, block[ 9], 14);
        GGG128(aaa, bbb, ccc, ddd, block[11], 12);
        GGG128(ddd, aaa, bbb, ccc, block[ 8], 13);
        GGG128(ccc, ddd, aaa, bbb, block[12],  5);
        GGG128(bbb, ccc, ddd, aaa, block[ 2], 14);
        GGG128(aaa, bbb, ccc, ddd, block[10], 13);
        GGG128(ddd, aaa, bbb, ccc, block[ 0], 13);
        GGG128(ccc, ddd, aaa, bbb, block[ 4],  7);
        GGG128(bbb, ccc, ddd, aaa, block[13],  5);

        /* parallel round 4 */
        FFF128(aaa, bbb, ccc, ddd, block[ 8], 15);
        FFF128(ddd, aaa, bbb, ccc, block[ 6],  5);
        FFF128(ccc, ddd, aaa, bbb, block[ 4],  8);
        FFF128(bbb, ccc, ddd, aaa, block[ 1], 11);
        FFF128(aaa, bbb, ccc, ddd, block[ 3], 14);
        FFF128(ddd, aaa, bbb, ccc, block[11], 14);
        FFF128(ccc, ddd, aaa, bbb, block[15],  6);
        FFF128(bbb, ccc, ddd, aaa, block[ 0], 14);
        FFF128(aaa, bbb, ccc, ddd, block[ 5],  6);
        FFF128(ddd, aaa, bbb, ccc, block[12],  9);
        FFF128(ccc, ddd, aaa, bbb, block[ 2], 12);
        FFF128(bbb, ccc, ddd, aaa, block[13],  9);
        FFF128(aaa, bbb, ccc, ddd, block[ 9], 12);
        FFF128(ddd, aaa, bbb, ccc, block[ 7],  5);
        FFF128(ccc, ddd, aaa, bbb, block[10], 15);
        FFF128(bbb, ccc, ddd, aaa, block[14],  8);

        /* combine results */
        ddd += cc + state[1];          /* final result for state[0] */
        state[1] = state[2] + dd + aaa;
        state[2] = state[3] + aa + bbb;
        state[3] = state[0] + bb + ccc;
        state[0] = ddd;
}

/********************************************************************/

void
RMD128_Update(RMD128_CTX *context, const uint8_t *data, size_t nbytes)
{
        uint32_t X[16];
        uint32_t ofs = 0;
        uint32_t i;
#ifdef WORDS_BIGENDIAN
        uint32_t j;
#endif

        _DIAGASSERT(context != NULL);
        _DIAGASSERT(data != NULL);

        /* update length[] */
        if (context->length[0] + nbytes < context->length[0])
                context->length[1]++;         /* overflow to msb of length */
        context->length[0] += nbytes;

        (void)memset(X, 0, sizeof(X));

        if ( context->buflen + nbytes < 64 )
        {
                (void)memcpy(context->bbuffer + context->buflen, data, nbytes);
                context->buflen += nbytes;
        }
        else
        {
                /* process first block */
                ofs = 64 - context->buflen;
                (void)memcpy(context->bbuffer + context->buflen, data, ofs);
#ifndef WORDS_BIGENDIAN
                (void)memcpy(X, context->bbuffer, sizeof(X));
#else
                for (j=0; j < 16; j++)
                        X[j] = BYTES_TO_DWORD(context->bbuffer + (4 * j));
#endif
                RMD128_Transform(context->state, X);
                nbytes -= ofs;

                /* process remaining complete blocks */
                for (i = 0; i < (nbytes >> 6); i++) {
#ifndef WORDS_BIGENDIAN
                        (void)memcpy(X, data + (64 * i) + ofs, sizeof(X));
#else
                        for (j=0; j < 16; j++)
                                X[j] = BYTES_TO_DWORD(data + (64 * i) + (4 * j) + ofs);
#endif
                        RMD128_Transform(context->state, X);
                }

                /*
                 * Put last bytes from data into context's buffer
                 */
                context->buflen = nbytes & 63;
                memcpy(context->bbuffer, data + (64 * i) + ofs, context->buflen);
        }
}

/********************************************************************/

void
RMD128_Final(uint8_t digest[16], RMD128_CTX *context)
{
        uint32_t i;
        uint32_t X[16];
#ifdef WORDS_BIGENDIAN
        uint32_t j;
#endif

        _DIAGASSERT(digest != NULL);
        _DIAGASSERT(context != NULL);

        /* append the bit m_n == 1 */
        context->bbuffer[context->buflen] = (uint8_t)'\200';

        (void)memset(context->bbuffer + context->buflen + 1, 0,
                63 - context->buflen);
#ifndef WORDS_BIGENDIAN
        (void)memcpy(X, context->bbuffer, sizeof(X));
#else
        for (j=0; j < 16; j++)
                X[j] = BYTES_TO_DWORD(context->bbuffer + (4 * j));
#endif
        if ((context->buflen) > 55) {
                /* length goes to next block */
                RMD128_Transform(context->state, X);
                (void)memset(X, 0, sizeof(X));
        }

        /* append length in bits */
        X[14] = context->length[0] << 3;
        X[15] = (context->length[0] >> 29) |
            (context->length[1] << 3);
        RMD128_Transform(context->state, X);

        if (digest != NULL) {
                for (i = 0; i < 16; i += 4) {
                        /* extracts the 8 least significant bits. */
                        digest[i]     =  context->state[i>>2];
                        digest[i + 1] = (context->state[i>>2] >>  8);
                        digest[i + 2] = (context->state[i>>2] >> 16);
                        digest[i + 3] = (context->state[i>>2] >> 24);
                }
        }
}

int RMD128_Equal(RMD128_CTX* pctx1, RMD128_CTX* pctx2) {
        return pctx1->buflen == pctx2->buflen
                && memcmp(pctx1->length, pctx2->length, sizeof(pctx1->length)) == 0
                && memcmp(pctx1->state, pctx2->state, sizeof(pctx1->state)) == 0
                && memcmp(pctx1->bbuffer, pctx2->bbuffer, sizeof(pctx1->bbuffer)) == 0;
}

/************************ end of file rmd128.c **********************/


char *
RMD128_End(RMD128_CTX *ctx, char *buf)
{
    size_t i;
    char *p = buf;
    uint8_t digest[16];
    static const char hex[]="0123456789abcdef";

    _DIAGASSERT(ctx != NULL);

    if (p == NULL && (p = malloc(41)) == NULL)
        return 0;

    RMD128_Final(digest,ctx);
    for (i = 0; i < 16; i++) {
        p[i + i] = hex[(uint32_t)digest[i] >> 4];
        p[i + i + 1] = hex[digest[i] & 0x0f];
    }
    p[i + i] = '\0';
    return(p);
}
