/*      $NetBSD: rmd128.h,v 1.2 2000/07/07 10:47:06 ad Exp $ */
/*      $RoughId: rmd128.h,v 1.3 2002/02/24 08:14:31 knu Exp $       */
/*      $Id: rmd128.h,v 1.2 2002/02/24 08:20:22 knu Exp $    */

/********************************************************************\
 *
 *      FILE:     rmd128.h
 *
 *      CONTENTS: Header file for a sample C-implementation of the
 *                RIPEMD-128 hash-function. 
 *      TARGET:   any computer with an ANSI C compiler
 *
 *      AUTHOR:   Antoon Bosselaers, ESAT-COSIC
 *      DATE:     1 March 1996
 *      VERSION:  1.0
 *
 *      Copyright (c) Katholieke Universiteit Leuven
 *      1996, All Rights Reserved
 *
\********************************************************************/

#include "rmdx.h"

#ifndef _RMD128_H_
#define _RMD128_H_

typedef unsigned int uint32_t;
typedef unsigned char uint8_t;
#include <stddef.h>

typedef struct {
        uint32_t       state[5];     /* state (ABCDE) */
        uint32_t       length[2];    /* number of bits */
        uint8_t                bbuffer[64];    /* overflow buffer */
        uint32_t       buflen;               /* number of chars in bbuffer */
} RMD128_CTX;

void    RMD128_Init(RMD128_CTX *);
void    RMD128_Transform(uint32_t[5], const uint32_t[16]);
void    RMD128_Update(RMD128_CTX *, const uint8_t *, size_t);
void    RMD128_Final(uint8_t[16], RMD128_CTX *);
int     RMD128_Equal(RMD128_CTX *, RMD128_CTX *);
char    *RMD128_End(RMD128_CTX *, char *);
char    *RMD128_Data(const uint8_t *, size_t, char *);

#define RMD128_BLOCK_LENGTH             64
#define RMD128_DIGEST_LENGTH            16
#define RMD128_DIGEST_STRING_LENGTH     (RMD128_DIGEST_LENGTH * 2 + 1)

#endif  /* !_RMD128_H_ */
