/*      $NetBSD: rmd160.h,v 1.2 2000/07/07 10:47:06 ad Exp $ */
/*      $RoughId: rmd160.h,v 1.3 2002/02/24 08:14:31 knu Exp $       */
/*      $Id: rmd160.h,v 1.2 2002/02/24 08:20:22 knu Exp $    */

/********************************************************************\
 *
 *      FILE:     rmd160.h
 *
 *      CONTENTS: Header file for a sample C-implementation of the
 *                RIPEMD-160 hash-function. 
 *      TARGET:   any computer with an ANSI C compiler
 *
 *      AUTHOR:   Antoon Bosselaers, ESAT-COSIC
 *      DATE:     1 March 1996
 *      VERSION:  1.0
 *
 *      Copyright (c) Katholieke Universiteit Leuven
 *      1996, All Rights Reserved
 *
\********************************************************************/

#include "rmdx.h"

#ifndef _RMD160_H_
#define _RMD160_H_

typedef struct {
        uint32_t       state[5];     /* state (ABCDE) */
        uint32_t       length[2];    /* number of bits */
        uint8_t                bbuffer[64];    /* overflow buffer */
        uint32_t       buflen;               /* number of chars in bbuffer */
} RMD160_CTX;

void    RMD160_Init(RMD160_CTX *);
void    RMD160_Transform(uint32_t[5], const uint32_t[16]);
void    RMD160_Update(RMD160_CTX *, const uint8_t *, size_t);
void    RMD160_Final(uint8_t[20], RMD160_CTX *);
int     RMD160_Equal(RMD160_CTX *, RMD160_CTX *);
char    *RMD160_End(RMD160_CTX *, char *);
char    *RMD160_Data(const uint8_t *, size_t, char *);

#define RMD160_BLOCK_LENGTH             64
#define RMD160_DIGEST_LENGTH            20
#define RMD160_DIGEST_STRING_LENGTH     (RMD160_DIGEST_LENGTH * 2 + 1)

#endif  /* !_RMD160_H_ */
