/**************************************************************
 *                                                            *
 *   Fichier : rmdx.h                                         *
 *   Header RIPEMD128-160                                     *
 *                                                            *
 **************************************************************/

#ifndef _RMDX_H_
#define _RMDX_H_

/* macro definitions */

/* collect four bytes into one word: */
#define BYTES_TO_DWORD(strptr)                  \
    (((uint32_t) *((strptr)+3) << 24) | \
    ((uint32_t) *((strptr)+2) << 16) |          \
    ((uint32_t) *((strptr)+1) <<  8) |          \
    ((uint32_t) *(strptr)))

/* ROL(x, n) cyclically rotates x over n bits to the left */
/* x must be of an unsigned 32 bits type and 0 <= n < 32. */
#define ROL(x, n)       (((x) << (n)) | ((x) >> (32-(n))))

/* the three basic functions F(), G() and H() */
#define F(x, y, z)      ((x) ^ (y) ^ (z))
#define G(x, y, z)      (((x) & (y)) | (~(x) & (z)))
#define H(x, y, z)      (((x) | ~(y)) ^ (z))
#define I(x, y, z)      (((x) & (z)) | ((y) & ~(z)))
#define J(x, y, z)      ((x) ^ ((y) | ~(z)))

#endif /* _RMDX_H_ */
