/**************************************************************
 *                                                            *
 *  Hash Finder                                               *
 *   Auteur : ordiman85                                       *
 *   Licence : Freeware & OpenSource                          *
 *                                                            *
 **************************************************************
 *                                                            *
 *   Fichier : main.h                                         *
 *   Header de main.c                                         *
 *                                                            *
 **************************************************************/


#define ProductTitle "Hash Finder v1.1.1"

/**************
 * Librairies *
 **************/

#include <windows.h>   // Fentres Windows
#include <stdio.h>   // Gestion des fichiers
#include "Hash/fnv.h"
#include "Hash/crc64.h"
#include "Hash/mdx.h"
#include "Hash/md2.h"
#include "Hash/md4.h"
#include "Hash/md5.h"
#include "Hash/rmdx.h"
#include "Hash/rmd128.h"
#include "Hash/rmd160.h"
#include "Hash/sha1.h"
#include "Hash/sha2.h"
#include "Hash/tiger.h"
#include "Hash/gost.h"
#include "Hash/haval.h"


/**************
 * Constantes *
 **************/

// ID des Hash
#define ID_Adler32 0

#define ID_CRC16 ID_Adler32+1
#define ID_CRC32 ID_CRC16+1
#define ID_CRC64 ID_CRC16+2

#define ID_ELF32 ID_CRC64+1

#define ID_FNV32 ID_ELF32+1
#define ID_FNV64 ID_FNV32+1

#define ID_GOST ID_FNV64+1

#define ID_HAVAL ID_GOST+1
#define ID_HAVAL3 ID_HAVAL+1
#define ID_HAVAL4 ID_HAVAL+2
#define ID_HAVAL5 ID_HAVAL+3

#define ID_MD2 ID_HAVAL5+1
#define ID_MD4 ID_MD2+1
#define ID_MD5 ID_MD2+2

#define ID_RMD128 ID_MD5+1
#define ID_RMD160 ID_RMD128+1

#define ID_SHA1 ID_RMD160+1
#define ID_SHA256 ID_SHA1+1
#define ID_SHA384 ID_SHA1+2
#define ID_SHA512 ID_SHA1+3

#define ID_TIGER ID_SHA512+1

#define ID_Par_Defaut ID_MD5

// Nom des hash
#define Nom_Adler32 "Adler32"
#define Nom_CRC16 "CRC-16"
#define Nom_CRC32 "CRC-32"
#define Nom_CRC64 "CRC-64"
#define Nom_ELF32 "ELF-32"
#define Nom_FNV32 "FNV-32"
#define Nom_FNV64 "FNV-64"
#define Nom_GOST "Gost"
#define Nom_HAVAL "HAVAL"
#define Nom_HAVAL3 "HAVAL-3"
#define Nom_HAVAL4 "HAVAL-4"
#define Nom_HAVAL5 "HAVAL-5"
#define Nom_MD2 "MD2"
#define Nom_MD4 "MD4"
#define Nom_MD5 "MD5"
#define Nom_RMD128 "RIPEMD-128"
#define Nom_RMD160 "RIPEMD-160"
#define Nom_SHA1 "SHA-1"
#define Nom_SHA256 "SHA-256"
#define Nom_SHA384 "SHA-384"
#define Nom_SHA512 "SHA-512"
#define Nom_TIGER "Tiger"
// Taille fentre
#define MarginLeft 50
#define MarginRight 80
#define ProductWindowClass "Hash_Finder_WinClass"
// Pour calculer le hash CRC64 avec la mthode ReHash
//#define ___ReHash_Code___


/**********************
 * Variables globales *
 **********************/

// Fentre
HWND hwnd;
HINSTANCE hinst;
char LigneCommande[1025];

// Gadgets
HWND Champ_Hash;
HWND Champ_Fichier;
HWND Selection_Type;
HWND Bouton_Parcourir;
HWND Bouton_Calculer;
HWND Bouton_Majuscules;
HWND Barre_Progression;

// Etat
char Etat = 0;

// Lecture fichier
unsigned long long Taille_Fichier;
unsigned long long Lecture;
FILE *fp;
#define Maj_Progression 150

// Hash
unsigned int Type_Hash;
unsigned char Global_Hash[1025];
long Taille_Tampon = 64*1024;


/****************
 * Dclarations *
 ****************/

void InitHash(); // Dbut du hash
void HashProc(); // Calcul du hash
void FinalHash(); // Finalisation du hash
void Annuler_Calcul(char Annuler); // Rinitialisation de l'tat initial (barre de progression, boutons...)
void SetFont(HWND gad); // Attribution de la police par dfaut Windows
char *GetWidgetText(HWND gad); // Retourne le texte d'un lment d'une fentre
long fsize(FILE* fd); // Taille de fichier
void Change_Majuscules(); // Mise en min/majucules du rsultat
char MinMaj(char c); // Fonction mettant en majuscules un caractre
char MajMin(char c); // Fonction mettant en minuscules un caractre
char OuvertureFichier(char* Fichier); // Ouvre le fichier pour hash
LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);


/******************
 * Contextes Hash *
 ******************/

// Contextes
unsigned int Calcul16;
unsigned long Calcul32;
unsigned long long Calcul64;
MD2_CTX MD2_Context;
MD4_CTX MD4_Context;
MD5_CTX MD5_Context;
RMD128_CTX RMD128_Context;
RMD160_CTX RMD160_Context;
SHA1_CTX SHA1_Context;
SHA256_CTX SHA256_Context;
SHA384_CTX SHA384_Context;
SHA512_CTX SHA512_Context;
TIGER_CTX TIGER_Context;
GOST_CTX GOST_Context;
HAVAL_CTX HAVAL_Context;


/**************************************************************
 * LIENS UTILES
 *************************************************************

http://www.users.zetnet.co.uk/hopwood/crypto/scan/md.html
http://serversniff.de/content.php?do=hashfile
http://serversniff.de/hash.php


/**************************************************************
 * TESTS HASH
 *************************************************************

"abc"
   CRC32 = 352441c2
         ->
   Adler32 = 024d0127
           ->
   MD2 = da853b0d3f88d99b30283a69e6ded6bb
       ->
   MD4 = a448017aaf21d8525fc10ae87aa6729d
       ->
   MD5 = 900150983cd24fb0d6963f7d28e17f72
       ->
   RIPEMD-128 = c14a12199c66e4ba84636b0f69144c77
              ->
   RIPEMD-160 = 8eb208f7e05d987a9b044a8e98c6b087f15a0bfc
              ->
   SHA-1 = a9993e364706816aba3e25717850c26c9cd0d89d
         ->
   SHA-256 = ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad
           ->
   SHA-384 = cb00753f45a35e8bb5a03d699ac65007272c32ab0eded1631a8b605a43ff5bed8086072ba1e7cc2358baeca134c825a7
           ->
   SHA-512 = ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f
           ->
   TIGER = 2aab1484e8c158f2bfb8c5ff41b57a525129131c957b5f93
         ->

/*************************************************************

CRC32 ("")
   00000000

CRC32 ("a")
   43beb7e8

CRC32 ("abc")
   352441c2

CRC32 ("ABC")
   a3830348

CRC32 ("123456789")
   cbf43926

CRC32 ("message digest")
   7f9d1520

CRC32 ("This is a string")
   876633f

CRC32 ("\uFFFE\u0000\u0001\u0002")
   b0e8eee5

/*************************************************************

Adler32 ("")
   00000001

Adler32 ("a")
   00620062

Adler32 ("abc")
   024d0127

Adler32 ("message digest")
   29750586

Adler32 ("abcdefghijklmnopqrstuvwxyz")
   90860b20

Adler32 ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
   8adb150c

/*************************************************************

MD2 ("")
   8350e5a3e24c153df2275c9f80692773

MD2 ("a")
   32ec01ec4a6dac72c0ab96fb34c0b5d1

MD2 ("abc")
   da853b0d3f88d99b30283a69e6ded6bb

MD2 ("message digest")
   ab4f496bfb2a530b219ff33031fe06b0

MD2 ("abcdefghijklmnopqrstuvwxyz")
   4e8ddff3650292ab5a4108c3aa47940b

MD2 ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
   da33def2a42df13975352846c30338cd

MD2 ("12345678901234567890123456789012345678901234567890123456789012345678901234567890")
   d5976f79d83d3a0dc9806c3c66f3efd8

/*************************************************************

MD4 ("")
   31d6cfe0d16ae931b73c59d7e0c089c0

MD4 ("a")
   bde52cb31de33e46245e05fbdbd6fb24

MD4 ("abc")
   a448017aaf21d8525fc10ae87aa6729d

MD4 ("message digest")
   d9130a8164549fe818874806e1c7014b

MD4 ("abcdefghijklmnopqrstuvwxyz")
   d79e1c308aa5bbcdeea8ed63df412da9

MD4 ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
   043f8582f241db351ce627e153e7f0e4

MD4 ("12345678901234567890123456789012345678901234567890123456789012345678901234567890")
   e33b4ddc9c38f2199c3e7b164fcc0536

/*************************************************************

MD5 ("")
   d41d8cd98f00b204e9800998ecf8427e

MD5 ("a")
   0cc175b9c0f1b6a831c399e269772661

MD5 ("abc")
   900150983cd24fb0d6963f7d28e17f72

MD5 ("message digest")
   f96b697d7cb7938d525a2f31aaf161d0

MD5 ("abcdefghijklmnopqrstuvwxyz")
   c3fcd3d76192e4007dfb496cca67e13b

MD5 ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
   d174ab98d277d9f5a5611c2c9f419d9f

MD5 ("12345678901234567890123456789012345678901234567890123456789012345678901234567890")
   57edf4a22be3c955ac49da2e2107b67a

/*************************************************************

RMD128 ("")
   cdf26213a150dc3ecb610f18f6b38b46

RMD128 ("a")
   86be7afa339d0fc7cfc785e72f578d33

RMD128 ("abc")
   c14a12199c66e4ba84636b0f69144c77

RMD128 ("message digest")
   9e327b3d6e523062afc1132d7df9d1b8

RMD128 ("abcdefghijklmnopqrstuvwxyz")
   fd2aa607f71dc8f510714922b371834e

RMD128 ("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
   a1aa0689d0fafa2ddc22e88b49133a06

RMD128 ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
   d1e959eb179c911faea4624c60c5c702

/*************************************************************

RMD160 ("")
   9c1185a5c5e9fc54612808977ee8f548b2258d31

RMD160 ("a")
   0bdc9d2d256b3ee9daae347be6f4dc835a467ffe

RMD160 ("abc")
   8eb208f7e05d987a9b044a8e98c6b087f15a0bfc

RMD160 ("message digest")
   5d0689ef49d2fae572b881b123a85ffa21595f36

RMD160 ("abcdefghijklmnopqrstuvwxyz")
   f71c27109c692c1b56bbdceb5b9d2865b3708dbc

RMD160 ("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
   12a053384a9c0c88e405a06c27dcf49ada62eb2b

RMD160 ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
   b0e20b6e3116640286ed3a87a5713079b21f5189

/*************************************************************

SHA1 ("")
   da39a3ee5e6b4b0d3255bfef95601890afd80709

SHA1 ("a")
   86f7e437faa5a7fce15d1ddcb9eaeaea377667b8

SHA1 ("abc")
   a9993e364706816aba3e25717850c26c9cd0d89d

SHA1 ("abcd")
   81fe8bfe87576c3ecb22426f8e57847382917acf

SHA1 ("message digest")
   c12252ceda8be8994d5fa0290a47231c1d16aae3

SHA1 ("abcdefghijklmnopqrstuvwxyz")
   32d10c7b8cf96570ca04ce37f2a19d84240d3a89

SHA1 ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
   761c457bf73b14d27e9e9265c46f4b4dda11f940

SHA1 ("12345678901234567890123456789012345678901234567890123456789012345678901234567890")
   50abf5706a150990a08b2c5ea40fa0e585554732

SHA1 ("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
   84983e441c3bd26ebaae4aa1f95129e5e54670f1

/*************************************************************

SHA256 ("abc")
   ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad

SHA256 ("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
   248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1

/*************************************************************

SHA384 ("abc")
   cb00753f45a35e8bb5a03d699ac65007272c32ab0eded1631a8b605a43ff5bed8086072ba1e7cc2358baeca134c825a7

SHA384 ("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
   3391fdddfc8dc7393707a65b1b4709397cf8b1d162af05abfe8f450de5f36bc6b0455a8520bc4e6f5fe95b1fe3c8452b

/*************************************************************

SHA512 ("abc")
   ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f

SHA512 ("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq")
   204a8fc6dda82f0a0ced7beb8e08a41657c16ef468b228a8279be331a703c33596fd15c13b1b07f9aa1d3bea57789ca031ad85c7a71dd70354ec631238ca3445

/*************************************************************

TIGER ("")
   3293ac630c13f0245f92bbb1766e16167a4e58492dde73f3

TIGER ("abc")
   2aab1484e8c158f2bfb8c5ff41b57a525129131c957b5f93

TIGER ("Tiger")
   dd00230799f5009fec6debc838bb6a27df2b9d6f110c7937

TIGER ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-")
   f71c8583902afb879edfe610f82c0d4786a3a534504486B5

TIGER ("ABCDEFGHIJKLMNOPQRSTUVWXYZ=abcdefghijklmnopqrstuvwxyz+0123456789")
   48ceeb6308b87d46e95d656112cdf18d97915f9765658957

TIGER ("Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham")
   8a866829040a410c729ad23f5ada711603b3cdd357e4c15e

TIGER ("Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham, proceedings of Fast Software Encryption 3, Cambridge.")
   ce55a6afd591f5ebac547ff84f89227f9331dab0b611c889

TIGER ("Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham, proceedings of Fast Software Encryption 3, Cambridge, 1996.")
   631abdd103eb9a3d245b6dfd4d77b257fc7439501d1568dd

TIGER ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-")
   c54034e5b43eb8005848a7e0ae6aac76e4ff590ae715fd25

TIGER (1 million de "a")
   6db0e2729cbead93d715c6a7d36302e9b3cee0d2bc314b41

/*************************************************************

*/
